@@ -32,6 +32,8 @@ class SetupAPI(BaseAPI):
             return self._validate_folders()
         elif action == 'current-config':
             return self._get_current_config()
+        elif action == 'existing-users':
+            return self._get_existing_users()
         else:
             bottle.abort(404, f"Setup action '{action}' not found")
     
@@ -181,6 +183,31 @@ class SetupAPI(BaseAPI):
         
         return {'validation_results': validation_results}
     
+    def _get_existing_users(self):
+        """Get existing users from the database for admin replacement option."""
+        try:
+            users = self.db.get_all_users()
+            user_list = []
+            
+            for user in users:
+                user_info = {
+                    'username': user.get('username', ''),
+                    'fullname': user.get('fullname', ''),
+                    'email': user.get('email', ''),
+                    'labname': user.get('labname', '')
+                }
+                user_list.append(user_info)
+            
+            return {
+                'result': 'success',
+                'users': user_list
+            }
+        except Exception as e:
+            return {
+                'result': 'error',
+                'message': f'Failed to fetch existing users: {str(e)}'
+            }
+    
     def _setup_basic_info(self):
         """Configure basic system information."""
         data = self.get_request_json()
@@ -511,6 +538,31 @@ class SetupAPI(BaseAPI):
                 # Store directly under 'mattermost' key (matching configuration.py structure)
                 self.config._settings['mattermost'] = mattermost_settings
             
+            # Update Slack settings (stored directly under 'slack' key)
+            slack_config = data.get('slack', {})
+            if slack_config:
+                current_slack = self.config._settings.get('slack', {})
+                
+                # Handle masked webhook URL - preserve existing URL if user didn't change it
+                submitted_webhook_url = slack_config.get('webhook_url', '')
+                if submitted_webhook_url == '***CONFIGURED***' and current_slack.get('webhook_url'):
+                    # User didn't change the masked webhook URL, preserve existing one
+                    actual_webhook_url = current_slack.get('webhook_url')
+                else:
+                    # User provided a new webhook URL (or cleared it)
+                    actual_webhook_url = submitted_webhook_url
+                
+                slack_settings = {
+                    'enabled': slack_config.get('enabled', False),
+                    'webhook_url': actual_webhook_url,
+                    'channel': slack_config.get('channel', ''),
+                    'use_webhook': slack_config.get('use_webhook', True),
+                    'use_manual_setup': slack_config.get('use_manual_setup', False)
+                }
+                
+                # Store directly under 'slack' key (matching configuration.py structure)
+                self.config._settings['slack'] = slack_settings
+            
             # Save configuration
             self.config.save()
             self.config.mark_setup_step_completed('notifications')
@@ -541,14 +593,19 @@ class SetupAPI(BaseAPI):
             
             # Update tunnel configuration with dual mode support
             tunnel_config = {
-                'enabled': data.get('enabled', False),
-                'mode': data.get('mode', 'custom'),  # 'custom' (free) or 'ethoscope_net' (paid)
+                'enabled': data.get('tunnel_enabled', False),
+                'mode': data.get('tunnel_mode', 'custom'),
                 'token': actual_token,
                 'node_id': data.get('node_id', 'auto'),
                 'domain': data.get('domain', 'ethoscope.net'),
                 'custom_domain': data.get('custom_domain', '')  # For custom domain mode
             }
             
+            # Extract authentication setting for separate configuration
+            auth_config = {
+                'enabled': data.get('authentication_enabled', False)
+            }
+            
             # Validate configuration based on mode
             if tunnel_config['enabled']:
                 if not tunnel_config['token']:
@@ -556,9 +613,13 @@ class SetupAPI(BaseAPI):
                 
                 if tunnel_config['mode'] == 'custom' and not tunnel_config['custom_domain']:
                     return {'result': 'error', 'message': 'Custom domain is required for free mode'}
+                
+                if not auth_config['enabled']:
+                    return {'result': 'error', 'message': 'Authentication must be enabled when remote access is active for security reasons'}
             
-            # Update configuration using the existing method
+            # Update configuration using the existing methods
             self.config.update_tunnel_config(tunnel_config)
+            self.config.update_authentication_config(auth_config)
             
             # Update the tunnel environment file if server is available
             if hasattr(self, 'server') and self.server:
@@ -583,6 +644,8 @@ class SetupAPI(BaseAPI):
                 return self._test_smtp(data.get('config', {}))
             elif test_type == 'mattermost':
                 return self._test_mattermost(data.get('config', {}))
+            elif test_type == 'slack':
+                return self._test_slack(data.get('config', {}))
             else:
                 return {'result': 'error', 'message': f'Unknown test type: {test_type}'}
                 
@@ -711,6 +774,75 @@ Ethoscope Node Setup Wizard
                 'message': f'Mattermost test failed: {str(e)}'
             }
     
+    def _test_slack(self, slack_config):
+        """Test Slack configuration by sending a test message."""
+        import requests
+        
+        try:
+            # Get configuration
+            webhook_url = slack_config.get('webhook_url', '')
+            channel = slack_config.get('channel', '')
+            
+            # Handle masked webhook URL - use the actual stored URL if masked value is sent
+            if webhook_url == '***CONFIGURED***':
+                current_slack = self.config._settings.get('slack', {})
+                webhook_url = current_slack.get('webhook_url', '')
+            
+            if not webhook_url:
+                return {
+                    'result': 'error',
+                    'message': 'Webhook URL is required for Slack testing'
+                }
+            
+            # Prepare test message with Block Kit formatting
+            message = {
+                'blocks': [
+                    {
+                        'type': 'section',
+                        'text': {
+                            'type': 'mrkdwn',
+                            'text': '🧪 *Ethoscope Test Message*'
+                        }
+                    },
+                    {
+                        'type': 'section',
+                        'text': {
+                            'type': 'mrkdwn',
+                            'text': 'This is a test message from the Ethoscope Node installation wizard. If you see this, your Slack configuration is working correctly!'
+                        }
+                    }
+                ],
+                'text': '🧪 Ethoscope Test Message - Slack notifications are working correctly!'
+            }
+            
+            # Add channel override if specified
+            if channel:
+                message['channel'] = channel
+            
+            # Send test message
+            response = requests.post(
+                webhook_url,
+                json=message,
+                timeout=10
+            )
+            
+            if response.status_code == 200:
+                return {
+                    'result': 'success',
+                    'message': 'Test message sent successfully to Slack'
+                }
+            else:
+                return {
+                    'result': 'error',
+                    'message': f'Slack test failed: HTTP {response.status_code} - {response.text}'
+                }
+                
+        except Exception as e:
+            return {
+                'result': 'error',
+                'message': f'Slack test failed: {str(e)}'
+            }
+    
     def _setup_virtual_sensor(self):
         """Configure virtual sensor settings."""
         try:
@@ -901,6 +1033,9 @@ Ethoscope Node Setup Wizard
                     'domain': 'ethoscope.net',
                     'custom_domain': ''
                 },
+                'authentication': {
+                    'enabled': False
+                },
                 'notifications': {
                     'smtp': {
                         'enabled': False,
@@ -916,6 +1051,13 @@ Ethoscope Node Setup Wizard
                         'server_url': '',
                         'bot_token': '',
                         'channel_id': ''
+                    },
+                    'slack': {
+                        'enabled': False,
+                        'webhook_url': '',
+                        'channel': '',
+                        'use_webhook': True,
+                        'use_manual_setup': False
                     }
                 },
                 'virtual_sensor': {
@@ -971,6 +1113,16 @@ Ethoscope Node Setup Wizard
             except Exception as e:
                 self.logger.warning(f"Could not load tunnel settings: {e}")
             
+            # Get authentication settings
+            try:
+                auth_config = self.config._settings.get('authentication', {})
+                if auth_config:
+                    config_data['authentication'].update({
+                        'enabled': auth_config.get('enabled', False)
+                    })
+            except Exception as e:
+                self.logger.warning(f"Could not load authentication settings: {e}")
+            
             # Get notification settings (from correct configuration paths)
             try:
                 # SMTP settings - stored directly under 'smtp' key
@@ -1003,6 +1155,21 @@ Ethoscope Node Setup Wizard
                         'bot_token': masked_token,  # Show masked token for UX, empty if none exists
                         'channel_id': mattermost_config.get('channel_id', '')
                     })
+                
+                # Slack settings - stored directly under 'slack' key
+                slack_config = self.config._settings.get('slack', {})
+                if slack_config:
+                    # Show masked webhook URL if one exists, empty if none configured
+                    existing_webhook_url = slack_config.get('webhook_url', '')
+                    masked_webhook_url = '***CONFIGURED***' if existing_webhook_url else ''
+                    
+                    config_data['notifications']['slack'].update({
+                        'enabled': slack_config.get('enabled', False),
+                        'webhook_url': masked_webhook_url,  # Show masked webhook URL for UX, empty if none exists
+                        'channel': slack_config.get('channel', ''),
+                        'use_webhook': slack_config.get('use_webhook', True),
+                        'use_manual_setup': slack_config.get('use_manual_setup', False)
+                    })
             except Exception as e:
                 self.logger.warning(f"Could not load notification settings: {e}")
             
